/*
 *	main.c
 *
 *	Cinepak AVI Player
 *
 *	* Wed Nov 21 19:08:08 JST 2007 Naoyuki Sawa
 *	- 1st [XB
 */
#include "app.h"

/****************************************************************************
 *	
 ****************************************************************************/

#ifdef USE_MUSIC
#include "instdef2.c"
#endif /*USE_MUSIC*/

#ifdef USE_SOUND
const void* sound_table[SOUND_COUNT];
#endif /*USE_SOUND*/

#ifdef USE_TEXTURE
TEXTURE texture_table[TEXTURE_COUNT];
#endif /*USE_TEXTURE*/

#ifdef USE_SPRITE
#define SPRITE_ SPRITE_INIT2
const SPRITE sprite_table[SPRITE_COUNT] = {
#include "sprite.h"
};
#undef SPRITE_
#endif /*USE_SPRITE*/

#define extern
#include "work.h"
#undef extern

/****************************************************************************
 *	
 ****************************************************************************/

typedef struct _AVIINFO {
	char fname[MAXFILENAME + 1];	/* t@C */
	int size;			/* t@CTCY */
} AVIINFO;

#define MAX_DISP	13		/* Sɕ\łt@C */

AVIINFO avi_info[MAXDIR];		/* AVIt@CXg */
int avi_count;				/* AVIt@C̐   */
int cursel;				/* ݂̑It@CCfNX */
int curtop;				/* ŏi\t@CCfNX */

static int
avi_info_compare(const void* _a, const void* _b)
{
	AVIINFO* a = (AVIINFO*)_a;
	AVIINFO* b = (AVIINFO*)_b;
	return strcmp(a->fname, b->fname);
}

/****************************************************************************
 *	main
 ****************************************************************************/

int
main()
{
	/* B */
	app_init();

	/* C[vB */
	app_main();

	return 0;
}

/****************************************************************************
 *	app_init
 ****************************************************************************/

void
app_init()
{
	/* ʓIȏB */
	pceAppSetProcPeriod(1000 / FPS);
#ifdef V_SCREEN
	pceLCDSetBuffer(_def_vbuff);
#else /*V_SCREEN*/
	pceLCDSetBuffer(vbuff);
#endif /*V_SCREEN*/
	pceLCDDispStart();
#ifdef QUICK_REPEAT
	pcePadSetTrigMode(PP_MODE_SINGLE);
#endif /*QUICK_REPEAT*/

	/* `B */
	surface.w = DISP_X;
	surface.h = DISP_Y;
	surface.vbuff = vbuff;
#ifdef USE_3D
	surface.zbuff = zbuff;
	render_init_3d(&render, &surface, 0, 0, 0, 0, FRONT, BACK, FOVY);
#else /*USE_3D*/
	render_init_2d(&render, &surface, 0, 0, 0, 0);
#endif /*USE_3D*/

#if defined(TRACE_ON) && (TRACE_ON)
	/* WG[o͂J܂B */
	stderr_open(9/*WG[o̓j^ڑ҂()*/);
#endif /*TRACE_ON*/

#ifdef USE_UFE
	/* USBt@CVXeG~[VJn܂B */
	ufe_setup(9/*PCt@CT[oڑ҂()*/);
#endif /*USE_UFE*/

#ifdef RESOURCE_FNAME
	/* pext@CYt\[X擾܂B */
	resource = resource_address(RESOURCE_FNAME);
#endif /*RESOURCE_FNAME*/

#ifdef TURBO
	/* NbNEEFCgؑւ܂B */
	turbo(TURBO);
#endif /*TURBO*/

#ifdef USE_DBG
	/* DBGjbgݒ肵܂B */
	dbg_init();
#endif /*USE_DBG*/

#ifdef USE_GC
	/* K[x[WRN^[܂B */
	gc_init();
#endif /*USE_GC*/

#ifdef USE_TCPIP
	/* TCP/IPvgRXC[gJn܂B */
	tcpip_start(0/*EthernethCoMobt@TCY()*/);
#endif /*USE_TCPIP*/

#ifdef LDIRECT_INTERVAL
	/* ؑցE16K\܂B */
	ldirect_init(LDIRECT_INTERVAL);
#endif /*LDIRECT_INTERVAL*/

#ifdef USE_MUSIC
	/* yCu܂B */
	music_loudness(1);
	loadInst(); /* uRandomism Zone(http://www8.plala.or.jp/randomism/)v́whFLbgver.2.10xgp */
	InitMusic();
#endif /*USE_MUSIC*/

	/* N[Abv֐o^B */
	atexit(app_exit);

#ifdef USE_2BIT_ADPCM
	/* 2-bit ADPCM DecoderN܂B */
	adpcm_init(ADPCM_CHANNELS);
#endif /*USE_2BIT_ADPCM*/

	/* TEh܂B */
#ifdef USE_SOUND
	{
#define SOUND_(name) { extern const unsigned char name[]; sound_table[SND_##name] = name; }
#define SOUND_RESOURCE_(name,fname) { sound_table[SND_##name] = fpk_address(resource, fname, NULL); }
#include "sound.h"
#undef SOUND_
#undef SOUND_RESOURCE_
	}
#endif /*USE_SOUND*/

	/* eNX`܂B */
#ifdef USE_TEXTURE
	{
#define TEXTURE_(name) extern const unsigned char name[];
#define TEXTURE_RESOURCE_(name,fname) const void* const name = fpk_address(resource, fname, NULL);
#include "texture.h"
#undef TEXTURE_
#undef TEXTURE_RESOURCE_
		texture_table_init(texture_table, TEXTURE_COUNT,
#define TEXTURE_(name) name,
#define TEXTURE_RESOURCE_(name,fname) name,
#include "texture.h"
#undef TEXTURE_
#undef TEXTURE_RESOURCE_
		NULL/*Ō','z邽߂̃_~[Btexture_table_init()ɑ΂Ă͖ӖłB*/);
	}
#endif /*USE_TEXTURE*/

#ifdef USE_SPRITE
	/* ȈՃXvCghCo܂B */
	sprite_init(&render, sprite_table, SPRITE_COUNT, texture_table, TEXTURE_COUNT);
#endif /*USE_SPRITE*/

#ifdef USE_VIDREC
	/* ʏo͂܂B */
	yield(); /* runŎsꍇ̎smFɉ邽߁AUSB@\̐؂ւ͂ɒx点 */
#ifdef LDIRECT_INTERVAL
	cdcacm_start(0, sizeof vbuff / 2, NULL, NULL);
#else /*LDIRECT_INTERVAL*/
	cdcacm_start(0, sizeof vbuff / 4, NULL, NULL);
#endif /*LDIRECT_INTERVAL*/
#endif /*USE_VIDREC*/

/*{{AvP[VL̏*/
	{
		int len;
		FILEINFO fi;
		FILE* fp;

		/* Cinepak AVIt@CT܂B */
		pceFileFindOpen(&fi);
		do {
			if(!pceFileFindNext(&fi)) {
				break; /* t@C */
			}
			len = strlen(fi.filename);
			if((len < 4) || (len > MAXFILENAME)) {
				continue; /* t@CZor */
			}
			if(strcasecmp(&fi.filename[len - 4], ".avi")) {
				continue; /* gqႤ */
			}
			fp = fopen(fi.filename, "rb");
			if(!fp) {
				continue; /* t@CJȂ(?) */
			}
			len = fread(_def_vbuff, 1, sizeof _def_vbuff, fp);
			fclose(fp);
			len -= 4;
			while(len >= 0) {
				if(!memcmp(&_def_vbuff[len], "cvid"/*Cinepak*/, 4)) {
					strcpy(avi_info[avi_count].fname, fi.filename);
					avi_info[avi_count].size = fi.length;
					avi_count++;
					break;
				}
				len--;
			}
		} while(avi_count < MAXDIR);
		pceFileFindClose(&fi);
		if(!avi_count) {
			ldirect_free();
			die("\n\n\n Cinepak AVI ̧ٶޱϾ SELECT  ޼خϽ");
		}

		/* t@CɃ\[g܂B */
		qsort(avi_info, avi_count, sizeof(AVIINFO), avi_info_compare);
	}
/*}}AvP[VL̏*/
}

/****************************************************************************
 *	app_exit
 ****************************************************************************/

void
app_exit()
{
#ifdef USE_VIDREC
	/* ʏo͂I܂B */
	cdcacm_stop();
#endif /*USE_VIDREC*/

#ifdef USE_MUSIC
	/* mɉyt~܂B
	 * TEh̒~̓CupceAppExit()ōsĂ̂ŁA
	 * ł͕Kv܂B
	 */
	StopMusic();
	music_loudness(0);
#endif /*USE_MUSIC*/

#ifdef LDIRECT_INTERVAL
	/* ؑցE16K\܂B */
	ldirect_free();
#endif /*LDIRECT_INTERVAL*/

#ifdef USE_TCPIP
	/* TCP/IPvgRXC[gI܂B */
	tcpip_stop();
#endif /*USE_TCPIP*/

#ifdef USE_GC
	/* K[x[WRN^[܂B */
	gc_free();
#endif /*USE_GC*/

#ifdef USE_DBG
	/* DBGjbg̐ݒ܂B */
	dbg_free();
#endif /*USE_DBG*/

#ifdef USE_UFE
	/* USBt@CVXeG~[VI܂B */
	ufe_stop();
#endif /*USE_UFE*/
}

/****************************************************************************
 *	update
 ****************************************************************************/

/* * Mon Nov 06 17:37:17 JST 2006 Naoyuki Sawa
 * - schedule()AʍXV𕪗܂B
 *   t[҂ʍXVsꍇ́Å֐ĂяoĂB
 */
void
update()
{
#ifdef V_SCREEN
	/* vbuff(c)_def_vbuff()ϊ */
	unsigned char *dst = &_def_vbuff[0];     /* ]̉ʂ͍ォ瑖Jn܂ */
	unsigned char *src = &vbuff[DISP_X - 1]; /* ]̏cʂ͉Eォ瑖Jn܂ */
	int x, y;
	for(x = DISP_X - 1; x >= 0; x--) {
		for(y = 0; y < DISP_Y; y++) {
			*dst = *src;
			dst += 1;      /* ʂEɑ܂ */
			src += DISP_X; /* cʂɑ܂ */
		}
		                            /* ʂ͊Ɏ̍sֈړĂ܂ */
		src -= DISP_X * DISP_Y + 1; /* cʂ̗̈ԏֈړ܂ */
	}
#endif /*V_SCREEN*/

	/* ʓ] */
	pceLCDTrans();
}

/****************************************************************************
 *	vidrec
 ****************************************************************************/

#ifdef USE_VIDREC
static void
vidrec()
{
#ifdef LDIRECT_INTERVAL
	static const unsigned char hdr[4] = { DISP_X,DISP_Y,4/*[bpp]*/,FPS };
#else /*LDIRECT_INTERVAL*/
	static const unsigned char hdr[4] = { DISP_X,DISP_Y,2/*[bpp]*/,FPS };
#endif /*LDIRECT_INTERVAL*/
	static int active;
	//
	unsigned char buf[128/**/]; /* (sizeof vbuff)/ÑTCYƂ邱 */
	const unsigned char* src;
	unsigned char* dst;

	if(cdcacm_recv(buf, 1) > 0) {
		switch(buf[0]) {
		case 0:
			active = 0; /* ʏo͒~ */
			break;
		case 1:
			active = 1; /* ʏo͊Jn */
			src = hdr;
			do {
				if(pcePadGetDirect() & PAD_SELECT) {
					exit(0); /* M҂Ɋ荞ŋI */
				}
				src += cdcacm_send(src, &hdr[sizeof hdr] - src);
			} while(src < &hdr[sizeof hdr]);
			break;
		}
	}
	if(active) {
		src = vbuff;
		do {
			dst = buf;
			do {
#ifdef LDIRECT_INTERVAL
				*dst++ = src[0]|(src[1]<<4); 
				src += 2;
#else /*LDIRECT_INTERVAL*/
				*dst++ = src[0]|(src[1]<<2)|(src[2]<<4)|(src[3]<<6);
				src += 4;
#endif /*LDIRECT_INTERVAL*/
			} while(dst < &buf[sizeof buf]);
			dst = buf;
			do {
				if(pcePadGetDirect() & PAD_SELECT) {
					exit(0); /* M҂Ɋ荞ŋI */
				}
				dst += cdcacm_send(dst, &buf[sizeof buf] - dst);
			} while(dst < &buf[sizeof buf]);
		} while(src < &vbuff[sizeof vbuff]);
	}
}
#endif /*USE_VIDREC*/

/****************************************************************************
 *	schedule
 ****************************************************************************/

/* * Fri Dec 15 00:49:15 JST 2006 Naoyuki Sawa
 * - schedule()֐ɁATEh|[YΉR[hǉ܂B
 *   ł͏肭삵Ă悤łAX[v̕Aɖ肪NȂmFłB
 * - A|[YԂ̂܂܁AȂ킿ATEh|[YԂ̂܂܂ŃAvP[VIĂ܂ꍇɂA
 *   CLiPCupceAppExit()pceWaveStop(1)ĂяoĂATEhԂZbĝŁAvłB
 */

void
schedule()
{
#ifdef USE_PAUSE
	{
		/* PAUSEbZ[Ẅ̃C[Wޔpobt@́A324oCgKvłB
		 * X^bNɎ̂͊댯łAstaticɎ̖̂ʂłB
		 * ŁAPAUSEɂ̂݃q[vmۂ邱Ƃɂ܂B
		 */
		//unsigned char save[PAUSE_H][PAUSE_W];
		unsigned char (*save)[PAUSE_W] = NULL/*x}*/;
		int pause = 0;
		/*{{2006/12/15:TEh|[YΉ*/
		int sound_pause = 0/*x}*/;
		/*}}2006/12/15:TEh|[YΉ*/
		int x, y;
		for(;;) {
#endif /*USE_PAUSE*/
			/* ʍXVB */
			update();
#ifdef USE_VIDREC
			/* ʏóB */
			vidrec();
#endif /*USE_VIDREC*/
			/* VXeB */
			yield();
			/* pbhԍXVB */
			joy = pcePadGet();
#ifdef V_SCREEN
			joy = (joy & ~(PAD_LF | PAD_RI | PAD_UP | PAD_DN | /* cʗp͕ϊ */
				       TRG_LF | TRG_RI | TRG_UP | TRG_DN))
				| (joy & PAD_LF ? PAD_UP : 0)
				| (joy & PAD_RI ? PAD_DN : 0)
				| (joy & PAD_UP ? PAD_RI : 0)
				| (joy & PAD_DN ? PAD_LF : 0)
				| (joy & TRG_LF ? TRG_UP : 0)
				| (joy & TRG_RI ? TRG_DN : 0)
				| (joy & TRG_UP ? TRG_RI : 0)
				| (joy & TRG_DN ? TRG_LF : 0);
#endif /*V_SCREEN*/

#ifdef USE_PAUSE
			/* PAUSEB */
			if(joy & TRG_START) {
				pause = !pause;	/* PAUSEԐ؂ւ */
				if(pause) {	/* PAUSE IN */
					/*{{2006/12/15:TEh|[YΉ*/
					ENTER_CS;
					sound_pause = fINT_EDMA.EHDM1;
					if(sound_pause) {
						fINT_EDMA.EHDM1 = 0;
					}
					LEAVE_CS;
					/*}}2006/12/15:TEh|[YΉ*/
					/* PAUSEbZ[W\̃C[WޔB */
					save = (unsigned char (*)[PAUSE_W])malloc(PAUSE_W * PAUSE_H); /* C[Wޔpobt@m */
					if(save == NULL) DIE();
					for(y = 0; y < PAUSE_H; y++) {
						for(x = 0; x < PAUSE_W; x++) {
							save[y][x] = vbuff[DISP_X * (PAUSE_Y + y) + (PAUSE_X + x)];
						}
					}
				} else {	/* PAUSE OUT */
					/*{{2006/12/15:TEh|[YΉ*/
					ENTER_CS;
					if(sound_pause) {
						fINT_EDMA.EHDM1 = 1;
					}
					LEAVE_CS;
					/*}}2006/12/15:TEh|[YΉ*/
					/* PAUSEbZ[W\̃C[W𕜋AB */
					for(y = 0; y < PAUSE_H; y++) {
						for(x = 0; x < PAUSE_W; x++) {
							vbuff[DISP_X * (PAUSE_Y + y) + (PAUSE_X + x)] = save[y][x];
						}
					}
					free(save); /* C[Wޔpobt@J */
				}
			}
			if(!pause) break;

			/* PAUSEbZ[W_ŕ\B */
			if(pceTimerGetCount() % 1000 < 750) {
				render_string_framed(&render, PAUSE_X, PAUSE_Y, PAUSE_STR, PAUSE_FONT, PAUSE_COLOR);
			} else {
				for(y = 0; y < PAUSE_H; y++) {
					for(x = 0; x < PAUSE_W; x++) {
						vbuff[DISP_X * (PAUSE_Y + y) + (PAUSE_X + x)] = save[y][x];
					}
				}
			}
		}
	}
#endif /*USE_PAUSE*/

#ifdef QUICK_REPEAT
	/* I[gs[gB */
	{
		static unsigned char rep[8]; /* I[gs[gJE^ */
		int i, mask;
		for(i = 0; i < 8; i++) {
			mask = 1 << i; /* PAD_* */
			if(QUICK_REPEAT_PADS & mask) {
				if(joy & mask) {
					if(rep[i] != 0xff) rep[i]++;
					if(rep[i] >= QUICK_REPEAT_DELAY) joy |= mask << 8; /* TRG_* */
				} else {
					rep[i] = 0;
				}
			}
		}
	}
#endif QUICK_REPEAT

	now++;	/* O[o^C}XV */
}

/****************************************************************************
 *	delay
 ****************************************************************************/

void
delay(int frame)
{
	while(frame > 0) {
		schedule();
		frame--;
	}
}

/****************************************************************************
 *	app_main
 ****************************************************************************/

void
app_main()
{
	int c;
	int i;
	int y;
	int pause;

	for(;;) {
		/*---------- I ----------*/
		for(;;) {
			schedule();
			surface_clear(&surface, 0);

			if(joy & TRG_SELECT) return; /* I */
			if(joy & TRG_A) break; /* ĐJn */

			/* IύX */
			if(joy & TRG_UP) cursel--;
			if(joy & TRG_DN) cursel++;
			if(joy & TRG_LF) cursel -= MAX_DISP;
			if(joy & TRG_RI) cursel += MAX_DISP;
			if(cursel < 0                      ) cursel = 0;
			if(cursel > avi_count - 1          ) cursel = avi_count - 1;
			if(cursel < curtop                 ) curtop = cursel                 ;
			if(cursel > curtop + (MAX_DISP - 1)) curtop = cursel - (MAX_DISP - 1);

			/* t@CXg\ */
			y = 8;
			for(i = curtop; i < curtop + MAX_DISP + 1; i++) {
				if(i > avi_count - 1) {
					break;
				}
				if(i == cursel) {
					render_rectangle_fill(&render, 0, y - 1, DISP_X, 6 + 1, 10);
					c = 0;
				} else {
					c = 10;
				}
				render_string(&render, 4 + 4 *  1, y, avi_info[i].fname, 0x82, c);
				render_printf(&render, 4 + 4 * 15, y, 0x82, c, "%8d bytes", avi_info[i].size);
				y += 6;
			}

			/* ^Cg\ */
			render_rectangle_fill(&render, 0, 0, DISP_X, 6 + 1, 15);
			render_string(&render, 0, 1,
			//       0123456789012345678901234567890123
				"======= Cinepak AVI Player =======",
				0x82, 0);
		}

		/*---------- I ----------*/
L_REWIND:
		pause = 0;
		cvid_init(avi_info[cursel].fname, 1);
		pceAppSetProcPeriod(1000 * cvid_driver.strh.dwScale / cvid_driver.strh.dwRate);
		for(;;) {
			schedule();

			if(joy & TRG_A) pause = !pause; /* |[Y */
			if(joy & (TRG_B | TRG_SELECT)) break; /* ĐI */
			if(joy & TRG_LF) { /* ߂ */
				//cvid_free(); /* cvid_init()IɌĂԂ̂ŕsv */
				goto L_REWIND;
			}
			if(joy & TRG_RI) { /*  */
				while(cvid_driver.loop && (pcePadGetDirect() == PAD_RI)) {
					update();
					cvid_update();
					surface_stretch(&surface, &cvid_driver.surface);
				}
			}
			if(!pause) { /* ʏĐ */
				cvid_update();
			}
			surface_stretch(&surface, &cvid_driver.surface);

			/* \ */
			if(!cvid_driver.loop) {
				render_string_framed(&render, -1, 66, "[a]:Iʂɖ߂", 0x80, 15);
				render_string_framed(&render, -1, 76, "[]:擪֊߂", 0x80, 15);
			} else if(pause) {
				render_string_framed(&render, -1, 46, "[`]:ꎞ~EĊJ", 0x80, 15);
				render_string_framed(&render, -1, 56, "[a]:Iʂɖ߂", 0x80, 15);
				render_string_framed(&render, -1, 66, "[]:(S)", 0x80, 15);
				render_string_framed(&render, -1, 76, "[]:擪֊߂", 0x80, 15);
			}

			/* ĐɃX^oCȂ悤ɁB(Ȓ[r[]łƂ͎vȂAÔ) */
			if(cvid_driver.loop && !pause) {
				pceAppActiveResponse(AAR_ACTIVE);
			}
		}
		cvid_free();
		pceAppSetProcPeriod(1000 / FPS);
	}
}
